@@ -17,6 +17,7 @@
 #include "utils/json.hpp"
 #include "Unit_cell/free_atom.hpp"
 #include "energy.hpp"
+#include "magnetization.hpp"
 #include "unit_cell_accessors.hpp"
 #include "make_sirius_comm.hpp"
 
@@ -98,6 +99,8 @@ std::string show_vec(const vector3d<T>& vec)
     return str;
 }
 
+void initialize_subspace(DFT_ground_state&, Simulation_context&);
+
 /* typedefs */
 template <typename T>
 using matrix_storage_slab = sddk::matrix_storage<T, sddk::matrix_storage_t::slab>;
@@ -162,6 +165,8 @@ PYBIND11_MODULE(py_sirius, m)
         .def_readonly("shiftk", &Parameters_input::shiftk_)
         .def_readonly("ngridk", &Parameters_input::ngridk_);
 
+    py::class_<Mixer_input>(m, "Mixer_input");
+
     py::class_<Communicator>(m, "Communicator");
 
     py::class_<Simulation_context>(m, "Simulation_context")
@@ -182,25 +187,32 @@ PYBIND11_MODULE(py_sirius, m)
         .def("full_potential", &Simulation_context::full_potential)
         .def("hubbard_correction", &Simulation_context::hubbard_correction)
         .def("fft", &Simulation_context::fft, py::return_value_policy::reference_internal)
+        .def("fft_coarse", &Simulation_context::fft_coarse, py::return_value_policy::reference_internal)
         .def("unit_cell", py::overload_cast<>(&Simulation_context::unit_cell, py::const_),
              py::return_value_policy::reference)
-        .def("pw_cutoff", &Simulation_context::pw_cutoff)
+        .def("pw_cutoff", py::overload_cast<>(&Simulation_context::pw_cutoff, py::const_))
+        .def("pw_cutoff", py::overload_cast<double>(&Simulation_context::pw_cutoff))
+        .def("gk_cutoff", py::overload_cast<>(&Simulation_context::gk_cutoff, py::const_))
+        .def("gk_cutoff", py::overload_cast<double>(&Simulation_context::gk_cutoff))
+        .def("aw_cutoff", py::overload_cast<>(&Simulation_context::aw_cutoff, py::const_))
+        .def("aw_cutoff", py::overload_cast<double>(&Simulation_context::aw_cutoff))
         .def("parameters_input", py::overload_cast<>(&Simulation_context::parameters_input, py::const_),
              py::return_value_policy::reference)
         .def("num_spin_dims", &Simulation_context::num_spin_dims)
         .def("num_mag_dims", &Simulation_context::num_mag_dims)
         .def("set_gamma_point", &Simulation_context::set_gamma_point)
-        .def("set_pw_cutoff", &Simulation_context::set_pw_cutoff)
         .def("update", &Simulation_context::update)
         .def("use_symmetry", py::overload_cast<>(&Simulation_context::use_symmetry, py::const_))
         .def("preferred_memory_t", &Simulation_context::preferred_memory_t)
+        .def("mixer_input", &Simulation_context::mixer_input)
         .def("comm", [](Simulation_context& obj) { return make_pycomm(obj.comm()); },
              py::return_value_policy::reference_internal)
         .def("comm_k", [](Simulation_context& obj) { return make_pycomm(obj.comm_k()); },
              py::return_value_policy::reference_internal)
         .def("comm_fft", [](Simulation_context& obj) { return make_pycomm(obj.comm_fft()); },
              py::return_value_policy::reference_internal)
-        .def("set_iterative_solver_tolerance", &Simulation_context::set_iterative_solver_tolerance);
+        .def("iterative_solver_tolerance", py::overload_cast<double>(&Simulation_context::iterative_solver_tolerance))
+        .def("iterative_solver_tolerance", py::overload_cast<>(&Simulation_context::iterative_solver_tolerance, py::const_));
 
     py::class_<Atom>(m, "Atom")
         .def("position", &Atom::position)
@@ -218,19 +230,30 @@ PYBIND11_MODULE(py_sirius, m)
         .def_property_readonly("num_atoms", [](const Atom_type& atype) { return atype.num_atoms(); });
 
     py::class_<Unit_cell>(m, "Unit_cell")
-        .def("add_atom_type",
-             static_cast<void (Unit_cell::*)(const std::string, const std::string)>(&Unit_cell::add_atom_type))
+        .def("add_atom_type", &Unit_cell::add_atom_type)
         .def("add_atom", py::overload_cast<const std::string, std::vector<double>>(&Unit_cell::add_atom))
         .def("atom", py::overload_cast<int>(&Unit_cell::atom), py::return_value_policy::reference)
         .def("atom_type", py::overload_cast<int>(&Unit_cell::atom_type), py::return_value_policy::reference)
         .def("lattice_vectors", &Unit_cell::lattice_vectors)
-        .def("set_lattice_vectors",
-             [](Unit_cell& obj, py::buffer l1, py::buffer l2, py::buffer l3) { set_lattice_vectors(obj, l1, l2, l3); })
+        .def(
+            "set_lattice_vectors",
+            [](Unit_cell& obj, py::buffer l1, py::buffer l2, py::buffer l3) { set_lattice_vectors(obj, l1, l2, l3); },
+            "l1"_a, "l2"_a, "l3"_a)
+        .def(
+            "set_lattice_vectors",
+            [](Unit_cell& obj, std::vector<double> l1, std::vector<double> l2, std::vector<double> l3) {
+                obj.set_lattice_vectors(vector3d<double>(l1), vector3d<double>(l2), vector3d<double>(l3));
+            },
+            "l1"_a, "l2"_a, "l3"_a)
         .def("get_symmetry", &Unit_cell::get_symmetry)
         .def("num_electrons", &Unit_cell::num_electrons)
         .def("num_valence_electrons", &Unit_cell::num_valence_electrons)
         .def("reciprocal_lattice_vectors", &Unit_cell::reciprocal_lattice_vectors)
-        .def("generate_radial_functions", &Unit_cell::generate_radial_functions);
+        .def("generate_radial_functions", &Unit_cell::generate_radial_functions)
+        .def("min_mt_radius", &Unit_cell::min_mt_radius)
+        .def("max_mt_radius", &Unit_cell::max_mt_radius)
+        .def("omega", &Unit_cell::omega)
+        .def("print_info", &Unit_cell::print_info);
 
     py::class_<z_column_descriptor>(m, "z_column_descriptor")
         .def_readwrite("x", &z_column_descriptor::x)
@@ -326,7 +349,6 @@ PYBIND11_MODULE(py_sirius, m)
         .def("generate", &Potential::generate)
         .def("symmetrize", &Potential::symmetrize)
         .def("fft_transform", &Potential::fft_transform)
-        .def("allocate", &Potential::allocate)
         .def("save", &Potential::save)
         .def("load", &Potential::load)
         .def("energy_vha", &Potential::energy_vha)
@@ -335,10 +357,13 @@ PYBIND11_MODULE(py_sirius, m)
         .def("PAW_total_energy", &Potential::PAW_total_energy)
         .def("PAW_one_elec_energy", &Potential::PAW_one_elec_energy);
 
-    py::class_<Density>(m, "Density")
+    py::class_<Field4D>(m, "Field4D")
+        .def("component", py::overload_cast<int>(&Field4D::component), py::return_value_policy::reference_internal);
+
+    py::class_<Density, Field4D>(m, "Density")
         .def(py::init<Simulation_context&>(), py::keep_alive<1, 2>(), "ctx"_a)
         .def("initial_density", &Density::initial_density)
-        .def("allocate", &Density::allocate)
+        .def("mixer_init", &Density::mixer_init)
         .def("check_num_electrons", &Density::check_num_electrons)
         .def("fft_transform", &Density::fft_transform)
         .def("mix", &Density::mix)
@@ -348,8 +373,21 @@ PYBIND11_MODULE(py_sirius, m)
         .def("generate_paw_loc_density", &Density::generate_paw_loc_density)
         .def("save", &Density::save)
         .def("check_num_electrons", &Density::check_num_electrons)
+        .def("f_pw_local",
+             [](py::object& obj, int i) -> py::array_t<complex_double> {
+
+               Density& wf   = obj.cast<Density&>();
+               auto& matrix_storage = wf.component(i).f_pw_local();
+               int nrows            = matrix_storage.size(0);
+               /* return underlying data as numpy.ndarray view */
+               return py::array_t<complex_double>({nrows},
+                                                  {1 * sizeof(complex_double)},
+                                                  matrix_storage.at(memory_t::host), obj);
+             },
+             py::keep_alive<0, 1>())
         .def("load", &Density::load);
 
+
     py::class_<Band>(m, "Band")
         .def(py::init<Simulation_context&>())
         .def("initialize_subspace", (void (Band::*)(K_point_set&, Hamiltonian&) const) & Band::initialize_subspace)
@@ -363,17 +401,25 @@ PYBIND11_MODULE(py_sirius, m)
         .def("total_energy", &DFT_ground_state::total_energy)
         .def("density", &DFT_ground_state::density, py::return_value_policy::reference)
         .def("find",
-             [](DFT_ground_state& dft, double potential_tol, double energy_tol, int num_dft_iter, bool write_state) {
-                 json js = dft.find(potential_tol, energy_tol, num_dft_iter, write_state);
+             [](DFT_ground_state& dft, double potential_tol, double energy_tol, double initial_tol, int num_dft_iter, bool write_state)
+             {
+                 json js = dft.find(potential_tol, energy_tol, initial_tol, num_dft_iter, write_state);
                  return pj_convert(js);
              },
-             "potential_tol"_a, "energy_tol"_a, "num_dft_iter"_a, "write_state"_a)
+             "potential_tol"_a, "energy_tol"_a, "initial_tol"_a, "num_dft_iter"_a, "write_state"_a)
+        .def("check_scf_density",
+             [](DFT_ground_state& dft)
+             {
+                 json js = dft.check_scf_density();
+                 return pj_convert(js);
+             })
         .def("k_point_set", &DFT_ground_state::k_point_set, py::return_value_policy::reference_internal)
         .def("hamiltonian", &DFT_ground_state::hamiltonian, py::return_value_policy::reference_internal)
         .def("potential", &DFT_ground_state::potential, py::return_value_policy::reference_internal)
         .def("forces", &DFT_ground_state::forces, py::return_value_policy::reference_internal)
         .def("stress", &DFT_ground_state::stress, py::return_value_policy::reference_internal)
-        .def("update", &DFT_ground_state::update);
+        .def("update", &DFT_ground_state::update)
+        .def("energy_kin_sum_pw", &DFT_ground_state::energy_kin_sum_pw);
 
     py::class_<K_point>(m, "K_point")
         .def("band_energy", py::overload_cast<int, int>(&K_point::band_energy, py::const_))
@@ -459,7 +505,7 @@ PYBIND11_MODULE(py_sirius, m)
                  }
              })
         .def("apply_ref",
-             [](Hamiltonian& hamiltonian, K_point& kp, Wave_functions& wf_out, Wave_functions& wf) {
+             [](Hamiltonian& hamiltonian, K_point& kp, Wave_functions& wf_out, Wave_functions& wf, std::shared_ptr<Wave_functions>& swf) {
                  int num_wf = wf.num_wf();
                  int num_sc = wf.num_sc();
                  if (num_wf != wf_out.num_wf() || wf_out.num_sc() != num_sc) {
@@ -472,6 +518,7 @@ PYBIND11_MODULE(py_sirius, m)
                      auto& mpd = ctx.mem_pool(memory_t::device);
                      for (int ispn = 0; ispn < num_sc; ++ispn) {
                          wf_out.pw_coeffs(ispn).allocate(mpd);
+                         wf.pw_coeffs(ispn).allocate(mpd);
                          wf.pw_coeffs(ispn).copy_to(memory_t::device, 0, num_wf);
                      }
                  }
@@ -485,12 +532,12 @@ PYBIND11_MODULE(py_sirius, m)
                  kp.beta_projectors().prepare();
                  if (!hamiltonian.ctx().gamma_point()) {
                      for (int ispn = 0; ispn < num_sc; ++ispn) {
-                         hamiltonian.apply_h_s<complex_double>(&kp, ispn, N, n, wf, &wf_out, nullptr);
+                         hamiltonian.apply_h_s<complex_double>(&kp, ispn, N, n, wf, &wf_out, swf.get());
                      }
                  } else {
                      std::cerr << "python module:: H applied at gamma-point\n";
                      for (int ispn = 0; ispn < num_sc; ++ispn) {
-                         hamiltonian.apply_h_s<double>(&kp, ispn, N, n, wf, &wf_out, nullptr);
+                         hamiltonian.apply_h_s<double>(&kp, ispn, N, n, wf, &wf_out, swf.get());
                      }
                  }
                  kp.beta_projectors().dismiss();
@@ -506,13 +553,14 @@ PYBIND11_MODULE(py_sirius, m)
                  }
                  #endif // __GPU
              },
-             "kpoint"_a, "wf_out"_a, "wf_in"_a)
+             "kpoint"_a, "wf_out"_a, "wf_in"_a, py::arg("swf_out")=nullptr)
         .def("_apply_ref_inner_prepare",
              // must be called BEFORE _apply_ref_inner
              [](Hamiltonian& hamiltonian) { hamiltonian.prepare(); })
         .def("_apply_ref_inner_dismiss",
              // must be called AFTER _apply_ref_inner
              [](Hamiltonian& hamiltonian) {
+                 hamiltonian.local_op().dismiss();
                  if (!hamiltonian.ctx().full_potential()) {
                      hamiltonian.dismiss();
                  }
@@ -520,7 +568,7 @@ PYBIND11_MODULE(py_sirius, m)
         .def("_apply_ref_inner",
              // this is the same as apply_ref, but skipping hamiltonian.prepare() which will deadlock unless called by
              // all ranks
-             [](Hamiltonian& hamiltonian, K_point& kp, Wave_functions& wf_out, Wave_functions& wf) {
+             [](Hamiltonian& hamiltonian, K_point& kp, Wave_functions& wf_out, Wave_functions& wf, std::shared_ptr<Wave_functions> swf) {
                  int num_wf = wf.num_wf();
                  int num_sc = wf.num_sc();
                  if (num_wf != wf_out.num_wf() || wf_out.num_sc() != num_sc) {
@@ -533,7 +581,13 @@ PYBIND11_MODULE(py_sirius, m)
                      auto& mpd = ctx.mem_pool(memory_t::device);
                      for (int ispn = 0; ispn < num_sc; ++ispn) {
                          wf_out.pw_coeffs(ispn).allocate(mpd);
+                         wf.pw_coeffs(ispn).allocate(mpd);
                          wf.pw_coeffs(ispn).copy_to(memory_t::device, 0, num_wf);
+                         if (swf) {
+                             swf->pw_coeffs(ispn).allocate(mpd);
+                             swf->pw_coeffs(ispn).copy_to(memory_t::device, 0, num_wf);
+                         }
+
                      }
                  }
                  #endif
@@ -545,28 +599,83 @@ PYBIND11_MODULE(py_sirius, m)
                  kp.beta_projectors().prepare();
                  if (!hamiltonian.ctx().gamma_point()) {
                      for (int ispn = 0; ispn < num_sc; ++ispn) {
-                         hamiltonian.apply_h_s<complex_double>(&kp, ispn, N, n, wf, &wf_out, nullptr);
+                         hamiltonian.apply_h_s<complex_double>(&kp, ispn, N, n, wf, &wf_out, swf.get());
                      }
                  } else {
                      std::cerr << "python module:: H applied at gamma-point\n";
                      for (int ispn = 0; ispn < num_sc; ++ispn) {
-                         hamiltonian.apply_h_s<double>(&kp, ispn, N, n, wf, &wf_out, nullptr);
+                         hamiltonian.apply_h_s<double>(&kp, ispn, N, n, wf, &wf_out, swf.get());
                      }
                  }
                  kp.beta_projectors().dismiss();
-                 hamiltonian.local_op().dismiss();
+                 //hamiltonian.local_op().dismiss();
                  hamiltonian.ctx().fft_coarse().dismiss();
                  #ifdef __GPU
                  if (is_device_memory(ctx.preferred_memory_t())) {
-                     for (int ispn = 0; ispn < num_sc; ++ispn)
+                     for (int ispn = 0; ispn < num_sc; ++ispn) {
                          wf_out.pw_coeffs(ispn).copy_to(memory_t::host, 0, n);
+                         if(swf) {
+                             swf->pw_coeffs(ispn).copy_to(memory_t::host, 0, n);
+                         }
+                     }
                  }
                  #endif // __GPU
-             });
+             }, "kpoint"_a, "wf_out"_a, "wf_in"_a, py::arg("swf_out")=nullptr)
+        .def("_apply_overlap_inner",
+             // this is the same as apply_ref, but skipping hamiltonian.prepare() which will deadlock unless called by
+             // all ranks
+             [](Hamiltonian& hamiltonian, K_point& kp, Wave_functions& wf_out, Wave_functions& wf) {
+                 int num_wf = wf.num_wf();
+                 int num_sc = wf.num_sc();
+                 if (num_wf != wf_out.num_wf() || wf_out.num_sc() != num_sc) {
+                     throw std::runtime_error(
+                         "Hamiltonian::apply_s (python bindings): num_sc or num_wf do not match");
+                 }
+                 #ifdef __GPU
+                 auto& ctx = hamiltonian.ctx();
+                 if (is_device_memory(ctx.preferred_memory_t())) {
+                     auto& mpd = ctx.mem_pool(memory_t::device);
+                     for (int ispn = 0; ispn < num_sc; ++ispn) {
+                         wf_out.pw_coeffs(ispn).allocate(mpd);
+                         wf.pw_coeffs(ispn).allocate(mpd);
+                         wf.pw_coeffs(ispn).copy_to(memory_t::device, 0, num_wf);
+                     }
+                 }
+                 #endif
+                 // hamiltonian.local_op().prepare(kp.gkvec_partition());
+                 // hamiltonian.ctx().fft_coarse().prepare(kp.gkvec_partition());
+                 kp.beta_projectors().prepare();
+                 if (!hamiltonian.ctx().gamma_point()) {
+                     for (int ispn = 0; ispn < num_sc; ++ispn) {
+                         //hamiltonian.apply_s<complex_double>(&kp, ispn, wf, wf_out);
+                         hamiltonian.apply_h_s<complex_double>(&kp, ispn, 0, num_wf, wf, nullptr, &wf_out);
+                     }
+                 } else {
+                     std::cerr << "python module:: H applied at gamma-point\n";
+                     for (int ispn = 0; ispn < num_sc; ++ispn) {
+                         hamiltonian.apply_h_s<double>(&kp, ispn, 0, num_wf, wf, nullptr, &wf_out);
+                         //hamiltonian.apply_s<double>(&kp, ispn, wf, wf_out);
+                     }
+                 }
+                 kp.beta_projectors().dismiss();
+                 //hamiltonian.local_op().dismiss();
+                 // hamiltonian.ctx().fft_coarse().dismiss();
+                 #ifdef __GPU
+                 if (is_device_memory(ctx.preferred_memory_t())) {
+                     for (int ispn = 0; ispn < num_sc; ++ispn)
+                         wf_out.pw_coeffs(ispn).copy_to(memory_t::host, 0, num_wf);
+                 }
+                 #endif // __GPU
+             }, "kpoint"_a, "wf_out"_a, "wf_in"_a);
 
     py::class_<Stress>(m, "Stress")
         .def(py::init<Simulation_context&, Density&, Potential&, Hamiltonian&, K_point_set&>())
         .def("calc_stress_total", &Stress::calc_stress_total, py::return_value_policy::reference_internal)
+        .def("calc_stress_har", &Stress::calc_stress_har, py::return_value_policy::reference_internal)
+        .def("calc_stress_ewald", &Stress::calc_stress_ewald, py::return_value_policy::reference_internal)
+        .def("calc_stress_xc", &Stress::calc_stress_xc, py::return_value_policy::reference_internal)
+        .def("calc_stress_kin", &Stress::calc_stress_kin, py::return_value_policy::reference_internal)
+        .def("calc_stress_vloc", &Stress::calc_stress_vloc, py::return_value_policy::reference_internal)
         .def("print_info", &Stress::print_info);
 
     py::class_<Free_atom>(m, "Free_atom")
@@ -611,11 +720,46 @@ PYBIND11_MODULE(py_sirius, m)
         .def_property_readonly("total", &Force::forces_total)
         .def("print_info", &Force::print_info);
 
+    py::class_<FFT3D_grid>(m, "FFT3D_grid")
+        .def_property_readonly("size", py::overload_cast<>(&FFT3D_grid::size, py::const_))
+        .def_property_readonly("shape", [](const FFT3D_grid& obj) -> std::array<int,3> {
+                return std::array<int,3>{obj.size(0), obj.size(1), obj.size(2)};
+            })
+        .def_property_readonly("grid_size", &FFT3D_grid::grid_size)
+        ;
+
+    py::class_<FFT3D, FFT3D_grid>(m, "FFT3D")
+        .def_property_readonly("comm", &FFT3D::comm)
+        .def_property_readonly("local_size", &FFT3D::local_size)
+        ;
+
     py::class_<matrix_storage_slab<complex_double>>(m, "MatrixStorageSlabC")
         .def("is_remapped", &matrix_storage_slab<complex_double>::is_remapped)
         .def("prime", py::overload_cast<>(&matrix_storage_slab<complex_double>::prime),
              py::return_value_policy::reference_internal);
 
+    py::class_<mdarray<complex_double, 1>>(m, "mdarray1c")
+        .def("on_device", &mdarray<complex_double, 1>::on_device)
+        .def("copy_to_host", [](mdarray<complex_double, 1>& mdarray) { mdarray.copy_to(memory_t::host); })
+        .def("__array__", [](py::object& obj) {
+                            mdarray<complex_double, 1>& arr = obj.cast<mdarray<complex_double, 1>&>();
+                            int nrows                       = arr.size(0);
+                            return py::array_t<complex_double>({nrows},
+                                                               {1 * sizeof(complex_double)},
+                                                               arr.at(memory_t::host), obj);
+                          });
+
+    py::class_<mdarray<double, 1>>(m, "mdarray1r")
+        .def("on_device", &mdarray<double, 1>::on_device)
+        .def("copy_to_host", [](mdarray<double, 1>& mdarray) { mdarray.copy_to(memory_t::host); })
+        .def("__array__", [](py::object& obj) {
+                            mdarray<double, 1>& arr = obj.cast<mdarray<double, 1>&>();
+                            int nrows                       = arr.size(0);
+                            return py::array_t<double>({nrows},
+                              {1 * sizeof(double)},
+                                                               arr.at(memory_t::host), obj);
+                          });
+
     py::class_<mdarray<complex_double, 2>>(m, "mdarray2c")
         .def("on_device", &mdarray<complex_double, 2>::on_device)
         .def("copy_to_host", [](mdarray<complex_double, 2>& mdarray) { mdarray.copy_to(memory_t::host); })
@@ -645,7 +789,8 @@ PYBIND11_MODULE(py_sirius, m)
 
     py::enum_<sddk::memory_t>(m, "MemoryEnum").value("device", memory_t::device).value("host", memory_t::host);
 
-    py::class_<Wave_functions>(m, "Wave_functions")
+    // use std::shared_ptr as holder type, this required by Hamiltonian.apply_ref, apply_ref_inner
+    py::class_<Wave_functions, std::shared_ptr<Wave_functions>>(m, "Wave_functions")
         .def(py::init<Gvec_partition const&, int, memory_t, int>(), "gvecp"_a, "num_wf"_a, "mem"_a, "num_sc"_a)
         .def("num_sc", &Wave_functions::num_sc)
         .def("num_wf", &Wave_functions::num_wf)
@@ -677,7 +822,7 @@ PYBIND11_MODULE(py_sirius, m)
                      }
                  }
                  for (int ispn = 0; ispn < wf.num_sc(); ispn++) {
-                     wf.copy_to(spin_idx(ispn), memory_t::device, 0, wf.num_wf());
+                     wf.copy_to(spin_range(ispn), memory_t::device, 0, wf.num_wf());
                  }
              })
         .def("copy_to_cpu",
@@ -690,7 +835,7 @@ PYBIND11_MODULE(py_sirius, m)
                  if (!is_on_device) {
                  } else {
                      for (int ispn = 0; ispn < wf.num_sc(); ispn++) {
-                         wf.copy_to(spin_idx(ispn), memory_t::host, 0, wf.num_wf());
+                         wf.copy_to(spin_range(ispn), memory_t::host, 0, wf.num_wf());
                      }
                  }
              })
@@ -705,6 +850,16 @@ PYBIND11_MODULE(py_sirius, m)
         .def("pw_coeffs_obj", py::overload_cast<int>(&Wave_functions::pw_coeffs, py::const_),
              py::return_value_policy::reference_internal);
 
+    py::class_<Smooth_periodic_function<complex_double>>(m, "CSmooth_periodic_function")
+        .def("pw", &Smooth_periodic_function<complex_double>::pw_array, py::return_value_policy::reference_internal)
+        .def("rg", &Smooth_periodic_function<complex_double>::rg_array, py::return_value_policy::reference_internal);
+
+    py::class_<Smooth_periodic_function<double>>(m, "RSmooth_periodic_function")
+        .def("pw", &Smooth_periodic_function<double>::pw_array, py::return_value_policy::reference_internal)
+        .def("rg", &Smooth_periodic_function<double>::rg_array, py::return_value_policy::reference_internal);
+
+    py::class_<Periodic_function<double>, Smooth_periodic_function<double>>(m, "RPeriodic_function");
+
     m.def("ewald_energy", &ewald_energy);
     m.def("set_atom_positions", &set_atom_positions);
     m.def("atom_positions", &atom_positions);
@@ -713,4 +868,14 @@ PYBIND11_MODULE(py_sirius, m)
     m.def("omp_get_num_threads", &omp_get_num_threads);
     m.def("make_sirius_comm", &make_sirius_comm);
     m.def("make_pycomm", &make_pycomm);
+    m.def("magnetization", &magnetization);
+    m.def("initialize_subspace", &initialize_subspace);
+}
+
+
+void initialize_subspace(DFT_ground_state& dft_gs, Simulation_context& ctx)
+{
+    auto& kset = dft_gs.k_point_set();
+    auto& hamiltonian = dft_gs.hamiltonian();
+    Band(ctx).initialize_subspace(kset, hamiltonian);
 }
