@@ -44,21 +44,21 @@ class Band // TODO: Band class is lightweight and in principle can be converted
     /// BLACS grid for distributed linear algebra operations.
     BLACS_grid const& blacs_grid_;
 
-    inline void solve_full_potential(K_point& kp__, Hamiltonian& hamiltonian__) const;
+    void solve_full_potential(K_point& kp__, Hamiltonian& hamiltonian__) const;
 
     /// Solve the first-variational (non-magnetic) problem with exact diagonalization.
     /** This is only used by the LAPW method. */
-    inline void diag_full_potential_first_variation_exact(K_point& kp__, Hamiltonian& hamiltonian__) const;
+    void diag_full_potential_first_variation_exact(K_point& kp__, Hamiltonian& hamiltonian__) const;
 
     /// Solve the first-variational (non-magnetic) problem with iterative Davidson diagonalization.
-    inline void diag_full_potential_first_variation_davidson(K_point& kp__, Hamiltonian& hamiltonian__) const;
+    void diag_full_potential_first_variation_davidson(K_point& kp__, Hamiltonian& hamiltonian__) const;
 
     /// Solve second-variational problem.
-    inline void diag_full_potential_second_variation(K_point& kp, Hamiltonian& hamiltonian__) const;
+    void diag_full_potential_second_variation(K_point& kp, Hamiltonian& hamiltonian__) const;
 
     /// Get singular components of the LAPW overlap matrix.
     /** Singular components are the eigen-vectors with a very small eigen-value. */
-    inline void get_singular_components(K_point& kp__, Hamiltonian& H__) const;
+    void get_singular_components(K_point& kp__, Hamiltonian& H__) const;
 
     /// Diagonalize a pseudo-potential Hamiltonian.
     template <typename T>
@@ -66,52 +66,18 @@ class Band // TODO: Band class is lightweight and in principle can be converted
 
     /// Exact (not iterative) diagonalization of the Hamiltonian.
     template <typename T>
-    inline void diag_pseudo_potential_exact(K_point* kp__, int ispn__, Hamiltonian& H__) const;
+    void diag_pseudo_potential_exact(K_point* kp__, int ispn__, Hamiltonian& H__) const;
 
     /// Iterative Davidson diagonalization.
     template <typename T>
-    inline int diag_pseudo_potential_davidson(K_point* kp__, Hamiltonian& H__) const;
+    int diag_pseudo_potential_davidson(K_point* kp__, Hamiltonian& H__) const;
 
     template <typename T>
-    inline std::vector<double> diag_S_davidson(K_point& kp__, Hamiltonian& H__) const;
-
-    /// RMM-DIIS diagonalization.
-    template <typename T>
-    inline void diag_pseudo_potential_rmm_diis(K_point* kp__, int ispn__, Hamiltonian& H__) const;
+    mdarray<double, 1> diag_S_davidson(K_point& kp__, Hamiltonian& H__) const;
 
+    ///// RMM-DIIS diagonalization.
     //template <typename T>
-    //inline void
-    //diag_pseudo_potential_chebyshev(K_point* kp__, int ispn__, Hamiltonian& H__, P_operator<T>& p_op__) const;
-
-    /// Auxiliary function used internally by residuals() function.
-    inline mdarray<double, 1> residuals_aux(K_point* kp__,
-                                            int ispn__,
-                                            int num_bands__,
-                                            std::vector<double>& eval__,
-                                            Wave_functions& hpsi__,
-                                            Wave_functions& opsi__,
-                                            Wave_functions& res__,
-                                            mdarray<double, 2>& h_diag__,
-                                            mdarray<double, 1>& o_diag__) const;
-
-    /// Compute preconditioned residuals
-    template <typename T>
-    inline int residuals(K_point* kp__,
-                         int ispn__,
-                         int N__,
-                         int num_bands__,
-                         std::vector<double>& eval__,
-                         std::vector<double>& eval_old__,
-                         dmatrix<T>& evec__,
-                         Wave_functions& hphi__,
-                         Wave_functions& ophi__,
-                         Wave_functions& hpsi__,
-                         Wave_functions& opsi__,
-                         Wave_functions& res__,
-                         mdarray<double, 2>& h_diag__,
-                         mdarray<double, 1>& o_diag__,
-                         double eval_tolerance__,
-                         double norm_tolerance__) const; //TODO: more documentation here
+    //void diag_pseudo_potential_rmm_diis(K_point* kp__, int ispn__, Hamiltonian& H__) const;
 
     /// Check wave-functions for orthonormalization.
     template <typename T>
@@ -185,28 +151,28 @@ class Band // TODO: Band class is lightweight and in principle can be converted
      *  for the subspace spanned by the wave-functions \f$ \phi_i \f$. The matrix is always returned
      *  in the CPU pointer because most of the standard math libraries start from the CPU. */
     template <typename T>
-    inline void set_subspace_mtrx(int N__,
-                                  int n__,
-                                  Wave_functions& phi__,
-                                  Wave_functions& op_phi__,
-                                  dmatrix<T>& mtrx__,
-                                  dmatrix<T>* mtrx_old__ = nullptr) const;
+    void set_subspace_mtrx(int N__,
+                           int n__,
+                           Wave_functions& phi__,
+                           Wave_functions& op_phi__,
+                           dmatrix<T>& mtrx__,
+                           dmatrix<T>* mtrx_old__ = nullptr) const;
 
     template <typename T>
-    inline int solve_pseudo_potential(K_point& kp__, Hamiltonian& hamiltonian__) const;
+    int solve_pseudo_potential(K_point& kp__, Hamiltonian& hamiltonian__) const;
 
     template <typename T>
     void check_residuals(K_point& kp__, Hamiltonian& H__) const;
 
     /// Solve \f$ \hat H \psi = E \psi \f$ and find eigen-states of the Hamiltonian.
-    inline void solve(K_point_set& kset__, Hamiltonian& hamiltonian__, bool precompute__) const;
+    void solve(K_point_set& kset__, Hamiltonian& hamiltonian__, bool precompute__) const;
 
     /// Initialize the subspace for the entire k-point set.
-    inline void initialize_subspace(K_point_set& kset__, Hamiltonian& hamiltonian__) const;
+    void initialize_subspace(K_point_set& kset__, Hamiltonian& hamiltonian__) const;
 
     /// Initialize the wave-functions subspace.
     template <typename T>
-    inline void initialize_subspace(K_point* kp__, Hamiltonian& hamiltonian__, int num_ao__) const;
+    void initialize_subspace(K_point* kp__, Hamiltonian& hamiltonian__, int num_ao__) const;
 
     static double& evp_work_count() // TODO: move counters to sim.ctx
     {
@@ -215,14 +181,6 @@ class Band // TODO: Band class is lightweight and in principle can be converted
     }
 };
 
-#include "residuals.hpp"
-#include "diag_full_potential.hpp"
-#include "diag_pseudo_potential.hpp"
-#include "davidson.hpp"
-#include "initialize_subspace.hpp"
-#include "solve.hpp"
-#include "set_subspace_mtrx.hpp"
-
 }
 
 #endif // __BAND_HPP__
